import com.dbcswc.fs.Connection;
import com.dbcswc.fs.ConnectionException;
import com.dbcswc.fs.File;
import com.dbcswc.fs.FileException;

//
// this program is an example of accessing DB/C type files (text and index)
// using the DB/C FS Java Client Programming Interface which allows for
// traditional (non-SQL) access to DB/C type data and index files
//
// to compile this program you need reference fs.jar in the compile-time classpath
//
// to run this program you need to set up and run DB/C FS on a computer
// that is accessible to this program via TCP/IP, and then run this
// program with reference to fs.jar in the run-time classpath
//
// this program assumes there is an empty data file with three indexes created like this:
//
// # create vendor -d
// # index vendor 1-6
// # index vendor vendor2 7-26 1-6
// # aimdex vendor 7-26 94-133
//
// the file layout for this demo vendor file is:
//
// number  char  6 - vendor number
// name    char 20 - vendor name
// addr1   char 20 - address line 1
// addr2   char 20 - address line 2
// city    char 15 - city
// state   char  2 - state
// zip     char 10 - zip
// contact char 40 - contact person
// phone   char 15 - phone number
//


public class Demo {

private String server = "localhost";  // change this to the IP address or DNS name of your DB/C FS server computer
private Connection connection;  // the connection to the DB/C FS server is persistent
private char[] buffer;  // record buffer

public static void main(String[] args) {
	(new Demo()).run();
}

public void run() {
//
// first let's make sure the DB/C FS server is running
// this isn't necessary, but it might be useful to log the greeting message
//
	try {
		System.out.println(Connection.getgreeting(server));
	}
	catch (ConnectionException e) {
		System.out.println("the server is unavailable: " + e.toString());
		return;
	}
//
// connect to the server
// be sure you have user=testuser and password=testpass specified in the dbcfspwd.cfg file
// and the dbd file demodb.dbd file is set up correctly
//
	try {
		connection = new Connection(server, "demodb", "testuser", "testpass");
	}
	catch (ConnectionException e) {
		System.out.println("unable to connect: " + e.toString());
		return;
	}
	System.out.println("connected");
//
// create the record buffer and the File instances that match the vendor file indexes
//
	buffer = new char[148];
	File vendor1 = new File(connection);
	vendor1.setindexfile("vendor");
	vendor1.setoptions(File.DATA | File.IFILEOPEN);
	vendor1.setrecsize(148);
	vendor1.setrecordbuffer(buffer);
	File vendor2 = new File(connection);
	vendor2.setindexfile("vendor2");
	vendor2.setoptions(File.DATA | File.IFILEOPEN);
	vendor2.setrecsize(148);
	vendor2.setrecordbuffer(buffer);
	File vendor3 = new File(connection);
	vendor3.setindexfile("vendor");
	vendor3.setoptions(File.DATA | File.AFILEOPEN);
	vendor3.setrecsize(148);
	vendor3.setrecordbuffer(buffer);
//
// open the File instances
//
	try {
		vendor1.open();
		vendor2.open();
		vendor3.open();
	}
	catch (FileException e) {
		System.out.println("error opening file: " + e.toString());
		return;
	}
//
// write three records, updating the second index and the aimdex
// first move each record into the record buffer
//
	System.out.println("writing records");
	String record = "000501ACME Vending        123 Any Street      Suite 100           Peoria         IL61602     Mickey Mantle                           309-555-1212   ";
	record.getChars(0, 148, buffer, 0);
	try {
		vendor1.writekey(148, "000501");
		vendor2.insertkey("ACME Vending        000501");
		vendor3.insertkeys(148);
	}
	catch (FileException e) {
		System.out.println("error while writing record: " + e.toString());
		return;
	}
	record = "000201SNPP, Inc.          6001 Oceanview Drive                    Springfield    MO45502     Carl Carlson                            217-555-1212   ";
	record.getChars(0, 148, buffer, 0);
	try {
		vendor1.writekey(148, "000201");
		vendor2.insertkey("SNPP, Inc.          000201");
		vendor3.insertkeys(148);
	}
	catch (FileException e) {
		System.out.println("error while writing record: " + e.toString());
		return;
	}
	record = "000101Best Widget Company 300 Main Street                         South Park     CO80440     Kenny McCormick                         719-555-1212   ";
	record.getChars(0, 148, buffer, 0);
	try {
		vendor1.writekey(148, "000101");
		vendor2.insertkey("Best Widget Company 000101");
		vendor3.insertkeys(148);
	}
	catch (FileException e) {
		System.out.println("error while writing record: " + e.toString());
		return;
	}
//	
// now read the records in vendor name order
// first set key sequential position before first record
//
	System.out.println("vendors in name order: ");
	try {
		vendor2.readkey("                          ");
		while (true) {
			int n1 = vendor2.readkeynext();
			if (n1 < 0) break;
			System.out.println(String.valueOf(buffer, 6, 20));
		}
	}
	catch (FileException e) {
		System.out.println("error while reading record key sequentially: " + e.toString());
		return;
	}
//	
// now find the records with a contact matching "Mick" using the aim index
//
	System.out.println("vendors with Mick contact: ");
	try {
		int n1 = vendor3.readaim("02FMick");
		while (n1 > 0) {
			System.out.println(String.valueOf(buffer, 6, 20));
			n1 = vendor3.readkeynext();
		}
	}
	catch (FileException e) {
		System.out.println("error while reading record throw aim index: " + e.toString());
		return;
	}
//
// close the files and disconnect from the server
//
	try {
		vendor1.close();
		vendor2.close();
		vendor3.close();
		connection.disconnect();
	}
	catch (Exception e) { }
	System.out.println("done");
}

}
